#ifndef DDTCORE_ONLINEUNPACK_H
#define	DDTCORE_ONLINEUNPACK_H
///////////////////////////////////////////////////////////////
// \file    OnlineUnpack.h
// \author  janzirn@fnal.gov
//
// \brief   This class does the unpack from the flat (binary) 
//          DAQ data to the DDT format
////////////////////////////////////////////////////////////////
#include "DDTBaseDataProducts/DAQHit.h"
#include "DDTBaseDataProducts/EventHeader.h"

#include "DAQChannelMap/DAQChannelMapConstants.h"

#include <stdint.h> // int64_t

// forward dectarations

namespace rawfileparser {
    class RawFileParser;
}

namespace novaddt{

  typedef struct {
    // See DocDB 4390
    uint32_t   fTriggerHeader_Source;             ///! Trigger source (see DAQDataFormats/TriggerDefines.h)
    uint32_t   fTriggerHeader_SourceID;           ///! Identifier of the source system
    uint32_t   fTriggerHeader_SourceSubID;        ///! Identifier of the subsystem or channel on source device
    uint64_t   fTriggerHeader_TriggerNumber;      ///! Universal trigger number
    uint64_t   fTriggerHeader_MasterTriggerNumber;
    uint64_t   fTriggerTime_GenTime;
    uint32_t   fTriggerTime_ClockSource;
    uint64_t   fTriggerTimingMarker_TimeStart;
    uint64_t   fTriggerTimingMarker_ExtractionStart;
    uint32_t   fTriggerRange_InfBit;
    uint32_t   fTriggerRange_TriggerLength;
    uint32_t   fTriggerMask_MCBit;
    uint32_t   fTriggerMask_TMod;
    uint32_t   fTriggerMask_TriggerTypesLow;
    uint32_t   fTriggerMask_TriggerTypesHigh;
    uint32_t   fTriggerMask_TriggerTypesHigh2;
    uint32_t   fTriggerMask_Prescale;

    int        fDetectorType;   ///! Detector type
    int        fNumberOfHits; ///! Number of DAQHits

  } UnpackedEvent;


  class OnlineUnpack{

  public:
    /// Constructor
    OnlineUnpack (const std::string* file_name=nullptr);
    /// Destructor
    ~OnlineUnpack ();

    /// Unpack the RawEvent
    bool ProcessRawEvent (void* rawevent_buffer, std::vector<novaddt::DAQHit>*, novaddt::EventHeader*);
    bool ProcessRawEvent (rawfileparser::RawFileParser &parser, std::vector<novaddt::DAQHit>*, novaddt::EventHeader*);

    uint32_t   getRunNumber()                   const { return fRawEvent.getRunNumber();     }
    uint32_t   getSubrunNumber()                const { return fRawEvent.getSubrun();        }
    uint64_t   getEventNumber()                 const { return fRawEvent.getEventNumber();   }
    int        getDetId()                       const { return fDetId;                       }
    uint32_t   getNumberOfNonemptyMicroblocks() const { return fNumberOfNonemptyMicroblocks; }

    bool       fFilterCorruptedNanoslices;  ///< Do we need to filter out Corrupted NanoSlices?

  private:
    // Common code to unpack the event
    bool ProcessRawEvent_(std::vector<novaddt::DAQHit>*, novaddt::EventHeader*);

    /// Try to get Detector ID from the file name
    bool getDetIDFromFileName(const std::string*);

    int                      fDetId;                      ///< Detector ID reading from Microblocks
    uint32_t                 fNumberOfNonemptyMicroblocks;
    daqdataformats::RawEvent fRawEvent;                   ///< Flat DAQ RawEvent
    novaddt::DAQHit          fHit;                        ///< Instance of DAQHit class
    UnpackedEvent            fUnpackedEventParameters;
    uint32_t                 fHitNumber;
    const std::string*       fCurrentFileName;            ///< Current file name

  };
 
}// end of namespace novaddt

#endif	/* DDTCORE_ONLINEUNPACK_H */

