/////////////////////////////////////////////////////////////////
// \file    HoughPeak.h
// \brief   Contains class definition for hough peak object
// \version $ID: $
// \author  edniner@fnal.gov
////////////////////////////////////////////////////////////////
#ifndef HOUGHPEAK_H
#define HOUGHPEAK_H

namespace novaddt{
  class HoughPeak
  {
  public:
    HoughPeak();
    ~HoughPeak();

#ifndef __GCCXML__

  public:
    HoughPeak(const float rho,
              const float theta,
              const float sigmarho,
              const float sigmatheta,
              const int   significance);
    HoughPeak(const HoughPeak& hp);

    //getters for the data members
    const float Rho          () const;
    const float Theta        () const;
    const float SigmaRho     () const;
    const float SigmaTheta   () const;
    const int   Significance () const;

    //setters for the data members
    void SetRho          (const float& r   );
    void SetTheta        (const float& t   );
    void SetSigmaRho     (const float& sr  );
    void SetSigmaTheta   (const float& st  );
    void SetSignificance (const int&   sig );
#endif

  private:
    float _rho;           ///< hough transform distance from origin
    float _theta;         ///< hough transform angle coordinate
    float _sigmarho;      ///< spread in rho
    float _sigmatheta;    ///< spread in theta
    int   _significance;  ///< number of hough points in peak
  };


  inline
  HoughPeak():
    _rho(0.0f),
    _theta(0.0f),
    _sigmarho(0.0f),
    _sigmatheta(0.0f),
    _significance(0)
  {
  }

#ifndef __GCCXML__

  inline
  HoughPeak(const float rho,
            const float theta,
            const float sigmarho,
            const float sigmatheta,
            const int   significance) :
    _rho          (rho),
    _theta        (theta),
    _sigmarho     (sigmarho),
    _sigmatheta   (sigmatheta),
    _significance (significance)
  {
  }

  inline
  HoughPeak(const HoughPeak& hp):
    _rho          (hp._rho),
    _theta        (hp._theta),
    _sigmarho     (hp._sigmarho),
    _sigmatheta   (hp._sigmatheta),
    _significance (hp._significance)
  {
  }
    
  inline const float Rho          () const { return _rho;          }
  inline const float Theta        () const { return _theta;        }
  inline const float SigmaRho     () const { return _sigmaRho;     }
  inline const float SigmaTheta   () const { return _sigmatheta;   }
  inline const int   Significance () const { return _significance; }
  
  //setters for the data members
  inline void SetRho          (const float& r   ) { _rho          = r;   }
  inline void SetTheta        (const float& t   ) { _theta        = t;   }
  inline void SetSigmaRho     (const float& sr  ) { _sigmarho     = sr;  }
  inline void SetSigmaTheta   (const float& st  ) { _sigmatheta   = st;  }
  inline void SetSignificance (const int&   sig ) { _significance = sig; }

#endif

}
#endif
