////////////////////////////////////////////////////////////////////////////
// \version $Id: Track.h,v 1.5 2012-11-20 20:26:04 brebel Exp $
//
// \brief Definition of Track object
//
// \author brebel@fnal.gov
//
////////////////////////////////////////////////////////////////////////////
#ifndef NOVADDT_TRACK_H
#define NOVADDT_TRACK_H

#include <DDTBaseDataProducts/BaseProducts.h>
#include <stdint.h>

namespace novaddt {

  class Track{
    
  public:
    
    Track();

  private:

    uint8_t  fView;     // 0: yz ; 1: xz
    float    fInverseBeta;     // reversed velocity projection
    float    fStartV;   // track start v
    float    fStartZ;   // track start z
    TDC      fStartT;   // the TDC of the "start" hit, may not be the earliest one 
    float    fEndV;     // track end x
    float    fEndZ;     // track end z
    TDC      fEndT;   // the TDC of the "start" hit, may not be the latest one 

#ifndef __GCCXML__
  public:

    explicit Track(
		   uint8_t  const& view,
		   float    const& inversebeta,
		   float    const& startv,
		   float    const& startz,
		   TDC      const& startt,
		   float    const& endv,
		   float    const& endz,
		   TDC      const& endt
		   );

    uint8_t  const& View()         const;
    float    const& InverseBeta()  const;
    float    const& StartV()       const;
    float    const& StartZ()       const;
    TDC      const& StartT()       const;
    float    const& EndV()         const;
    float    const& EndZ()         const;
    TDC      const& EndT()         const;

#endif

  };

#ifndef __GCCXML__

  inline uint8_t  const& Track::View()   const { return fView;       }
  inline float    const& Track::InverseBeta()  const { return fInverseBeta;      }
  inline float    const& Track::StartV()   const { return fStartV;      }
  inline float    const& Track::StartZ()   const { return fStartZ;      }
  inline TDC      const& Track::StartT()   const { return fStartT;      }
  inline float    const& Track::EndV()     const { return fEndV;      }
  inline float    const& Track::EndZ()     const { return fEndZ;      }
  inline TDC      const& Track::EndT()     const { return fEndT;      }
  
#endif 

}
#endif //NOVADDT_TRACK_H
