////////////////////////////////////////////////////////////////////////
// Class:       HighADCFilter
// Module Type: producer
// File:        HighADCFilter_module.cc
//
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Persistency/Common/Assns.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Event.h"

#include "DDTBaseDataProducts/BaseProducts.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/DAQHit.h"
#include "DDTBaseDataProducts/GroupedHitList.h"

#include <string>
#include <algorithm>

namespace novaddt {
  class HighADCFilter;
}

class novaddt::HighADCFilter : public art::EDFilter{
public:
  explicit HighADCFilter(fhicl::ParameterSet const & p);
  virtual ~HighADCFilter();
  GroupedHitList<TDC> MakeSlice(HitList hits);
  virtual bool filter(art::Event & e);

private:
  std::string _hitslabel;
  std::string _instance;

  int _ADCmin, _ADCmax;
  // Declare member data here.

};


novaddt::HighADCFilter::HighADCFilter(fhicl::ParameterSet const & p)
:
_hitslabel (p.get< std::string >("hits_label")),
_instance  (p.get< std::string >("instance"  )),
_ADCmin    (p.get< int >("ADC_min"  )),
_ADCmax    (p.get< int >("ADC_max"  ))
// Initialize member data here.
{
  // Call appropriate Produces<>() functions here.
  produces<HitList>("HEHits");
}

novaddt::HighADCFilter::~HighADCFilter()
{
  // Clean up dynamic memory and other resources here.
}

bool novaddt::HighADCFilter::filter(art::Event & e)
{
  art::Handle<novaddt::HitList> hits;
  e.getByLabel(_hitslabel, _instance, hits);
  HitList hl;

  for (auto hit: *hits) {
  //  std::cout<<hit.ADC().val<<std::endl;
    if (hit.ADC().val>_ADCmin && hit.ADC().val<_ADCmax) 
      hl.emplace_back(hit);
  }

  std::unique_ptr<HitList> product(new HitList(hl));
  e.put(std::move(product),"HEHits");
  return true;
}


DEFINE_ART_MODULE(novaddt::HighADCFilter)
