////////////////////////////////////////////////////////////////////////
// Class:       RemoveNoise
// Module Type: producer
// File:        RemoveNoise_module.cc
//
// Generated at Wed Oct 24 09:51:33 2012 by Evan Niner using artmod
// from art v1_01_01.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Event.h"

#include "DDTBaseDataProducts/BaseProducts.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/DAQHit.h"
#include "DDTBaseDataProducts/GroupedHitList.h"

namespace novaddt {
  class RemoveNoise;
}

class novaddt::RemoveNoise : public art::EDFilter{
public:
  explicit RemoveNoise(fhicl::ParameterSet const & p);
  virtual ~RemoveNoise();

  virtual bool filter(art::Event & e);
  GroupedHitList<TDC> CleanSlice(art::Handle<novaddt::GroupedHitList<novaddt::TDC> >  &gh);


private:

  // Declare member data here.
  std::string _ghlabel;
  std::string _instance;
  unsigned int fMinHits;

};


novaddt::RemoveNoise::RemoveNoise(fhicl::ParameterSet const & p)
:
_ghlabel  (p.get< std::string >("gh_label")),
_instance (p.get< std::string >("instance")),
fMinHits  (p.get< int         >("MinHits" ))
// Initialize member data here.
{
  // Call appropriate Produces<>() functions here.
  produces<GroupedHitList<TDC>>("TDCSliceClean");
}

novaddt::RemoveNoise::~RemoveNoise()
{
  // Clean up dynamic memory and other resources here.
}

bool novaddt::RemoveNoise::filter(art::Event & e)
{
  art::Handle<novaddt::GroupedHitList<novaddt::TDC> > ghits;
  e.getByLabel(_ghlabel, _instance, ghits);

  //make a grouped hitlist out of hits after time slicing
  std::unique_ptr< GroupedHitList<TDC> >product(new GroupedHitList<TDC>);
  *product = CleanSlice(ghits);
  //put the hits into the event for later use
  e.put(std::move(product),"TDCSliceClean");

  return true;
}

novaddt::GroupedHitList<novaddt::TDC> novaddt::RemoveNoise::CleanSlice(art::Handle<novaddt::GroupedHitList<novaddt::TDC> > &gh)
{
  std::vector<size_t> boundaries; //new list of boundary points for timeslices
  HitList cleanHits;  //hitlist for groups above noise threshold

  //add boundary at start of list, this is required
  boundaries.push_back(0);
  
  for(unsigned i=0; i<gh->n_groups(); ++i){
    if (gh->group_size(i) < fMinHits) continue;
    
    HitList temp = gh->get(i);
    //if (cleanHits.size() != 0) boundaries.push_back((temp.begin())->TDC().val);
    cleanHits.insert(cleanHits.end(),temp.begin(),temp.end());
    boundaries.push_back(std::distance(cleanHits.begin(),cleanHits.end()));
  }

  return GroupedHitList<TDC>(cleanHits.begin(), cleanHits.end(), boundaries);
}


DEFINE_ART_MODULE(novaddt::RemoveNoise)
