////////////////////////////////////////////////////////////////////////
// Class:       SpaceSlice
// Module Type: producer
// File:        SpaceSlice_module.cc
//
// Generated at Wed Oct 24 09:51:33 2012 by Evan Niner using artmod
// from art v1_01_01.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Event.h"

#include "DDTBaseDataProducts/BaseProducts.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/DAQHit.h"
#include "DDTBaseDataProducts/GroupedHitList.h"
#include "DDTBaseDataProducts/CompareDAQHit.h"

namespace novaddt {
  class SpaceSlice;
}

class novaddt::SpaceSlice : public art::EDFilter{
public:
  explicit SpaceSlice(fhicl::ParameterSet const & p);
  virtual ~SpaceSlice();

  virtual bool filter(art::Event & e);

private:

  // Declare member data here.
  std::string _ghlabel;
  std::string _instance;
  int fMaxZGap;
  unsigned int fMinHits;

};


novaddt::SpaceSlice::SpaceSlice(fhicl::ParameterSet const & p)
:
_ghlabel  (p.get< std::string >("gh_label")),
_instance (p.get< std::string >("instance")),
fMaxZGap  (p.get< int         >("MaxZGap" )),
fMinHits  (p.get< int         >("MinHits" ))
// Initialize member data here.
{
  // Call appropriate Produces<>() functions here.
  produces<std::vector<HitList>>("TimeSpaceSlice");
}

novaddt::SpaceSlice::~SpaceSlice()
{
  // Clean up dynamic memory and other resources here.
}

bool  novaddt::SpaceSlice::filter(art::Event & e)
{

  art::Handle<novaddt::GroupedHitList<novaddt::TDC> > ghits;
  e.getByLabel(_ghlabel, _instance, ghits);

  //make a grouped hitlist out of hits after time slicing
  std::unique_ptr< std::vector<HitList> >product(new std::vector<HitList>);

  for(unsigned int i=0; i<ghits->n_groups(); ++i){
    HitList temp = ghits->get(i);
    if (temp.size()<1) continue;
    std::sort(temp.begin(), temp.end(), CompareDAQHit<Plane>());

    auto itStart = temp.begin();
    auto itEnd   = temp.begin();
    /*
    int xhits = 0;
    int yhits = 0;
    */
    while (itEnd != temp.end()-1) {
      const int endPlane = (*itEnd).Plane().val;
      ++itEnd;
      const int nextPlane = (*itEnd).Plane().val;

      if (nextPlane - endPlane > fMaxZGap) {
        if (itEnd - itStart > fMinHits) {
	  HitList frag(itStart, itEnd);
	  product->emplace_back(frag);
	}
        itStart = itEnd;
      }      
    }

    if (itEnd-itStart > fMinHits) {
      HitList frag(itStart, itEnd);
      product->emplace_back(frag);
 
    }
    /*
    std::cout<<"xhits: "<<xhits<<std::endl;
    std::cout<<"yhits: "<<yhits<<std::endl;
    */
  }

  //put the hits into the event for later use
  e.put(std::move(product),"TimeSpaceSlice");

  return true;
}


DEFINE_ART_MODULE(novaddt::SpaceSlice)
