////////////////////////////////////////////////////////////////////////
// Class:       HoughTracker
// Module Type: filter
// File:        HoughTracker_module.cc
//
// Generated at Thu Jan 10 15:01:30 2013 by Zukai Wang using artmod
// from art v1_02_06.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Persistency/Common/Assns.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Event.h"

#include "DDTBaseDataProducts/CompareDAQHit.h"
#include "DDTBaseDataProducts/BaseProducts.h"
#include "DDTBaseDataProducts/Boundary.h"
#include "DDTBaseDataProducts/BoundaryList.h"
#include "DDTBaseDataProducts/HitList.h"
#include "DDTBaseDataProducts/DAQHit.h"
#include "DDTBaseDataProducts/HoughPoint.h"
#include "DDTBaseDataProducts/GroupedHitList.h"
#include "DDTBaseDataProducts/Track.h"
#include "DDTUtilities/AssociationUtil.h"
#include "PatRec/HoughTrackMaker.h"

#include <tbb/tbb.h>

#include <vector>
#include <string>
#include <stdint.h>
#include <set>
#include <cmath>
#include <chrono>
#include <memory>

using namespace std::chrono;


namespace novaddt {
    class HoughTracker;
}

class novaddt::HoughTracker : public art::EDFilter 
{
public:
  explicit HoughTracker(fhicl::ParameterSet const & p);
  virtual ~HoughTracker();
  bool filter(art::Event & e) override;

  void endJob() override;
private:
  std::string     sliceLabel_;
  std::string     sliceInstance_;
  bool            parallel_;

  fhicl::ParameterSet fhicl_parameters_;
};

novaddt::HoughTracker::HoughTracker(fhicl::ParameterSet const & p)
  : sliceLabel_            (p.get<std::string>("slice_label"))
  , sliceInstance_         (p.get<std::string>("slice_instance"))
  , parallel_              (p.get<bool>("parallel"))
  , fhicl_parameters_(p)
{
  produces<std::vector<Track> >();
  produces<std::vector<HitList> >();
  produces<art::Assns<Track, HitList> >();
  produces<art::Assns<Track, HitList> >("toSlices");
}

novaddt::HoughTracker::~HoughTracker()
{
}

bool novaddt::HoughTracker::filter(art::Event & e)
{
  std::unique_ptr<art::Assns<novaddt::Track, novaddt::HitList> > 
    assn(new art::Assns<novaddt::Track, novaddt::HitList>);
  
  std::unique_ptr<art::Assns<novaddt::Track, novaddt::HitList> > 
    assn_b(new art::Assns<novaddt::Track, novaddt::HitList>);

  art::Handle<std::vector<novaddt::HitList>> slice_handle;
  e.getByLabel(sliceLabel_, sliceInstance_, slice_handle);

  HoughTrackMaker htm(fhicl_parameters_);

  art::PtrVector<novaddt::HitList> slices;
  for (unsigned index = 0; index < slice_handle->size(); ++index)
  {
    art::Ptr<novaddt::HitList> slice(slice_handle, index);
    slices.push_back(slice);
  }

  if (parallel_)
    tbb::parallel_reduce(br_t(slices.cbegin(), slices.cend()), htm);
  else
    htm.do_serial(slices);

  htm.create_associations(*this, e, *assn, *assn_b);
  htm.move_collections_into_event(e);
  e.put(std::move(assn));
  e.put(std::move(assn_b), "toSlices");

  return true;
}

void novaddt::HoughTracker::endJob()
{
}

DEFINE_ART_MODULE(novaddt::HoughTracker)
