////////////////////////////////////////////////////////////////////////
// Class:       SlowMonopoleTrigger
// Module Type: filter
// File:        SlowMonopoleTrigger_module.cc
//
// Generated at Mon Jun  1 21:39:22 2015 by Martin Frank using artmod
// from cetpkgsupport v1_08_05.
////////////////////////////////////////////////////////////////////////

#include <art/Framework/Core/EDFilter.h>
#include <art/Framework/Core/ModuleMacros.h>
#include <art/Framework/Principal/Event.h>
#include <art/Framework/Principal/Handle.h>
#include <art/Framework/Principal/Run.h>
#include <art/Framework/Principal/SubRun.h>
#include <art/Utilities/InputTag.h>
#include <fhiclcpp/ParameterSet.h>
#include <messagefacility/MessageLogger/MessageLogger.h>

#include <memory>

#include "SlowMonopoleTrigger/Trigger.h"

namespace novaddt
{
  class SlowMonopoleTrigger;
}


class novaddt::SlowMonopoleTrigger : public art::EDFilter
{
public:
  explicit SlowMonopoleTrigger(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  SlowMonopoleTrigger(SlowMonopoleTrigger const &) = delete;
  SlowMonopoleTrigger(SlowMonopoleTrigger &&) = delete;
  SlowMonopoleTrigger & operator = (SlowMonopoleTrigger const &) = delete;
  SlowMonopoleTrigger & operator = (SlowMonopoleTrigger &&) = delete;

  // Required functions.
  bool filter(art::Event & e) override;


private:
  std::string input_hits_label_, input_hits_instance_;
  smt::Trigger smt_;
};



novaddt::SlowMonopoleTrigger::SlowMonopoleTrigger
(fhicl::ParameterSet const & p) :
  input_hits_label_   (p.get<std::string>("input_hits_label")),
  input_hits_instance_(p.get<std::string>("input_hits_instance"))
{
  smt_ = smt::Trigger(p);

  produces<std::vector<novaddt::TriggerDecision> >();
}



bool novaddt::SlowMonopoleTrigger::filter(art::Event & e)
{
  std::unique_ptr<std::vector<novaddt::TriggerDecision> >
    trigger_decisions(new std::vector<novaddt::TriggerDecision>());
  
  art::Handle<novaddt::HitList> hits;
  e.getByLabel(input_hits_label_, input_hits_instance_, hits);

  bool trigger = smt_.run_algorithm(hits);
  if (trigger)
  {
    auto tds = smt_.trigger_decisions();
    trigger_decisions->assign(tds.begin(), tds.end());
  }

  e.put(std::move(trigger_decisions));
  return trigger;
}



DEFINE_ART_MODULE(novaddt::SlowMonopoleTrigger)
